@@ -66,6 +66,12 @@ GType gabble_bytestream_factory_get_type (void);
   (G_TYPE_INSTANCE_GET_CLASS ((obj), GABBLE_TYPE_BYTESTREAM_FACTORY,\
                               GabbleBytestreamFactoryClass))
 
+typedef struct {
+    gchar *jid;
+    gchar *host;
+    gchar *port;
+} GabbleSocks5Proxy;
+
 typedef void (* GabbleBytestreamFactoryNegotiateReplyFunc) (
     GabbleBytestreamIface *bytestream, const gchar *stream_id, LmMessage *msg,
     GObject *object, gpointer user_data);
@@ -80,7 +86,8 @@ GabbleBytestreamMuc *gabble_bytestream_factory_create_muc (
 GabbleBytestreamIface *gabble_bytestream_factory_create_from_method (
     GabbleBytestreamFactory *self, const gchar *stream_method,
     TpHandle peer_handle, const gchar *stream_id, const gchar *stream_init_id,
-    const gchar *peer_resource, GabbleBytestreamState state);
+    const gchar *peer_resource, const gchar *self_jid,
+    GabbleBytestreamState state);
 
 LmMessage *gabble_bytestream_factory_make_stream_init_iq (
     const gchar *full_jid, const gchar *stream_id, const gchar *profile);
@@ -99,6 +106,9 @@ gboolean gabble_bytestream_factory_negotiate_stream (
 
 gchar *gabble_bytestream_factory_generate_stream_id (void);
 
+GSList *gabble_bytestream_factory_get_socks5_proxies (
+    GabbleBytestreamFactory *self);
+
 G_END_DECLS
 
 #endif /* #ifndef __BYTESTREAM_FACTORY_H__ */
