@@ -58,6 +58,7 @@ enum
   PROP_STATE,
   PROP_PROTOCOL,
   PROP_FACTORY,
+  PROP_SELF_JID,
   LAST_PROPERTY
 };
 
@@ -71,10 +72,12 @@ struct _GabbleBytestreamMultiplePrivate
   GabbleBytestreamState state;
   gchar *peer_jid;
   GabbleBytestreamFactory *factory;
+  gchar *self_full_jid;
 
   /* List of (gchar *) containing the NS of a stream method */
   GList *fallback_stream_methods;
   GabbleBytestreamIface *active_bytestream;
+  gboolean read_blocked;
 
   gboolean dispose_has_run;
 };
@@ -132,6 +135,7 @@ gabble_bytestream_multiple_finalize (GObject *object)
   g_free (priv->stream_init_id);
   g_free (priv->peer_resource);
   g_free (priv->peer_jid);
+  g_free (priv->self_full_jid);
 
   G_OBJECT_CLASS (gabble_bytestream_multiple_parent_class)->finalize (object);
 }
@@ -178,6 +182,9 @@ gabble_bytestream_multiple_get_property (GObject *object,
       case PROP_FACTORY:
         g_value_set_object (value, priv->factory);
         break;
+      case PROP_SELF_JID:
+        g_value_set_string (value, priv->self_full_jid);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
         break;
@@ -224,6 +231,10 @@ gabble_bytestream_multiple_set_property (GObject *object,
       case PROP_FACTORY:
         priv->factory = g_value_get_object (value);
         break;
+      case PROP_SELF_JID:
+        g_free (priv->self_full_jid);
+        priv->self_full_jid = g_value_dup_string (value);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
         break;
@@ -262,6 +273,8 @@ gabble_bytestream_multiple_constructor (GType type,
   else
     priv->peer_jid = g_strdup (jid);
 
+  g_assert (priv->self_full_jid != NULL);
+
   return obj;
 }
 
@@ -324,6 +337,15 @@ gabble_bytestream_multiple_class_init (
       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_FACTORY,
       param_spec);
+
+  param_spec = g_param_spec_string (
+      "self-jid",
+      "Our self jid",
+      "Either a contact full jid or a muc jid",
+      NULL,
+      G_PARAM_CONSTRUCT_ONLY  | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_SELF_JID,
+      param_spec);
 }
 
 /*
@@ -492,6 +514,15 @@ bytestream_state_changed_cb (GabbleBytestreamIface *bytestream,
   g_object_set (self, "state", state, NULL);
 }
 
+static void
+bytestream_write_blocked_cb (GabbleBytestreamIface *bytestream,
+                             gboolean blocked,
+                             gpointer self)
+{
+  /* Forward signal */
+  g_signal_emit_by_name (G_OBJECT (self), "write-blocked", blocked);
+}
+
 static void
 bytestream_connection_error_cb (GabbleBytestreamIface *failed,
                                 gpointer user_data)
@@ -511,6 +542,8 @@ bytestream_connection_error_cb (GabbleBytestreamIface *failed,
       bytestream_data_received_cb, self);
   g_signal_handlers_disconnect_by_func (failed,
       bytestream_state_changed_cb, self);
+  g_signal_handlers_disconnect_by_func (failed,
+      bytestream_write_blocked_cb, self);
 
   /* We don't have to unref it because the reference is kept by the
    * factory */
@@ -547,16 +580,26 @@ bytestream_activate_next (GabbleBytestreamMultiple *self)
 
   priv->active_bytestream = gabble_bytestream_factory_create_from_method (
       priv->factory, stream_method, priv->peer_handle, priv->stream_id,
-      priv->stream_init_id, priv->peer_resource, priv->state);
+      priv->stream_init_id, priv->peer_resource, priv->self_full_jid,
+      priv->state);
+
+  /* Methods have already been checked so this shouldn't fail */
+  g_assert (priv->active_bytestream != NULL);
 
   g_free (stream_method);
 
+  /* block the new bytestream if needed */
+  gabble_bytestream_iface_block_reading (priv->active_bytestream,
+      priv->read_blocked);
+
   g_signal_connect (priv->active_bytestream, "connection-error",
       G_CALLBACK (bytestream_connection_error_cb), self);
   g_signal_connect (priv->active_bytestream, "data-received",
       G_CALLBACK (bytestream_data_received_cb), self);
   g_signal_connect (priv->active_bytestream, "state-changed",
       G_CALLBACK (bytestream_state_changed_cb), self);
+  g_signal_connect (priv->active_bytestream, "write-blocked",
+      G_CALLBACK (bytestream_write_blocked_cb), self);
 }
 
 /*
@@ -596,6 +639,23 @@ gabble_bytestream_multiple_has_stream_method (GabbleBytestreamMultiple *self)
   return (g_list_length (priv->fallback_stream_methods) != 0);
 }
 
+static void
+gabble_bytestream_multiple_block_reading (GabbleBytestreamIface *iface,
+                                          gboolean block)
+{
+  GabbleBytestreamMultiple *self = GABBLE_BYTESTREAM_MULTIPLE (iface);
+  GabbleBytestreamMultiplePrivate *priv =
+    GABBLE_BYTESTREAM_MULTIPLE_GET_PRIVATE (self);
+
+  if (priv->read_blocked == block)
+    return;
+
+  priv->read_blocked = block;
+
+  g_assert (priv->active_bytestream != NULL);
+  gabble_bytestream_iface_block_reading (priv->active_bytestream, block);
+}
+
 static void
 bytestream_iface_init (gpointer g_iface,
                        gpointer iface_data)
@@ -606,4 +666,5 @@ bytestream_iface_init (gpointer g_iface,
   klass->send = gabble_bytestream_multiple_send;
   klass->close = gabble_bytestream_multiple_close;
   klass->accept = gabble_bytestream_multiple_accept;
+  klass->block_reading = gabble_bytestream_multiple_block_reading;
 }
