@@ -188,9 +188,25 @@ add_to_geoloc_node (const gchar *tp_name,
   LocationMapping *mapping;
   gchar *str = NULL;
 
+  /* Map "language" to the xml:lang attribute. */
+  if (!tp_strdiff (tp_name, "language"))
+    {
+      if (G_VALUE_TYPE (value) != G_TYPE_STRING)
+        {
+          g_set_error (err, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+              "expecting string for language value, but got %s",
+                  G_VALUE_TYPE_NAME (value));
+          return FALSE;
+        }
+
+      lm_message_node_set_attribute (
+          geoloc, "xml:lang", g_value_get_string (value));
+      return TRUE;
+    }
+
   mapping = g_hash_table_lookup (tp_to_xmpp, tp_name);
-  if (mapping == NULL &&
-      tp_strdiff (tp_name, "language"))
+
+  if (mapping == NULL)
     {
       DEBUG ("Unknown location key: %s ; skipping", (const gchar *) tp_name);
       /* We don't raise a D-Bus error if the key is unknown to stay backward
@@ -198,15 +214,12 @@ add_to_geoloc_node (const gchar *tp_name,
       return TRUE;
     }
 
-  if (mapping != NULL && G_VALUE_TYPE (value) != mapping->type)
+  if (G_VALUE_TYPE (value) != mapping->type)
     {
-#define ERROR_MSG "'%s' is supposed to be of type %s but is %s",\
-          (const char *) tp_name, g_type_name (mapping->type),\
-          G_VALUE_TYPE_NAME (value)
-
-      DEBUG (ERROR_MSG);
-      g_set_error (err, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT, ERROR_MSG);
-#undef ERROR_MSG
+      g_set_error (err, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+          "'%s' is supposed to be of type %s but is %s",
+          (const char *) tp_name, g_type_name (mapping->type),
+          G_VALUE_TYPE_NAME (value));
       return FALSE;
     }
 
@@ -225,14 +238,6 @@ add_to_geoloc_node (const gchar *tp_name,
   else if (G_VALUE_TYPE (value) == G_TYPE_STRING)
     {
       str = g_value_dup_string (value);
-
-      if (!tp_strdiff (tp_name, "language"))
-        {
-          /* Set the xml:lang */
-          lm_message_node_set_attribute (geoloc, "xml:lang", str);
-          g_free (str);
-          return TRUE;
-        }
     }
   else
     /* Keys and their type have been checked */
@@ -281,6 +286,7 @@ location_set_location (TpSvcConnectionInterfaceLocation *iface,
       if (!add_to_geoloc_node ((const gchar *) key, (GValue *) value, geoloc,
             &err))
         {
+          DEBUG ("%s", err->message);
           dbus_g_method_return_error (context, err);
           g_error_free (err);
           goto out;
