@@ -1355,6 +1355,10 @@ data_received_cb (GabbleBytestreamIface *stream,
           TP_SVC_CHANNEL_TYPE_FILE_TRANSFER (self),
           TP_FILE_TRANSFER_STATE_COMPLETED,
           TP_FILE_TRANSFER_STATE_CHANGE_REASON_NONE);
+
+      if (gibber_transport_buffer_is_empty (self->priv->transport))
+        gibber_transport_disconnect (self->priv->transport);
+
       return;
     }
 
@@ -1408,8 +1412,17 @@ gabble_file_transfer_channel_accept_file (TpSvcChannelTypeFileTransfer *iface,
                                           DBusGMethodInvocation *context)
 {
   GabbleFileTransferChannel *self = GABBLE_FILE_TRANSFER_CHANNEL (iface);
+  TpBaseConnection *base_conn = (TpBaseConnection *) self->priv->connection;
   GError *error = NULL;
 
+  if (self->priv->initiator == base_conn->self_handle)
+    {
+      g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+          "Channel is not an incoming transfer");
+      dbus_g_method_return_error (context, error);
+      return;
+    }
+
   if (self->priv->state != TP_FILE_TRANSFER_STATE_PENDING)
     {
       g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
@@ -1491,6 +1504,15 @@ gabble_file_transfer_channel_provide_file (
       return;
     }
 
+  if (self->priv->state != TP_FILE_TRANSFER_STATE_PENDING &&
+      self->priv->state != TP_FILE_TRANSFER_STATE_ACCEPTED)
+    {
+      g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+        "State is not pending or accepted; cannot provide file");
+      dbus_g_method_return_error (context, error);
+      return;
+    }
+
   if (self->priv->socket_address != NULL)
     {
       g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
@@ -1668,6 +1690,9 @@ transport_buffer_empty_cb (GibberTransport *transport,
   /* Buffer is empty so we can unblock the buffer if it was blocked */
   DEBUG ("file transfer buffer is empty. Unblock the bytestream");
   gabble_bytestream_iface_block_reading (self->priv->bytestream, FALSE);
+
+  if (self->priv->state > TP_FILE_TRANSFER_STATE_OPEN)
+    gibber_transport_disconnect (transport);
 }
 
 /*
