@@ -112,6 +112,7 @@ enum
   PROP_CHANNEL_PROPERTIES,
   PROP_INITIAL_AUDIO,
   PROP_INITIAL_VIDEO,
+  PROP_IMMUTABLE_STREAMS,
   /* TP properties (see also below) */
   PROP_NAT_TRAVERSAL,
   PROP_STUN_SERVER,
@@ -145,6 +146,7 @@ typedef struct {
     gulong removed_id;
     gchar *name;
     const gchar *nat_traversal;
+    gboolean initial;
 } StreamCreationData;
 
 struct _delayed_request_streams_ctx {
@@ -181,12 +183,14 @@ gabble_media_channel_init (GabbleMediaChannel *self)
 static void session_state_changed_cb (GabbleJingleSession *session,
     GParamSpec *arg1, GabbleMediaChannel *channel);
 static void session_terminated_cb (GabbleJingleSession *session,
-    gboolean local_terminator, TpChannelGroupChangeReason reason,
+    gboolean local_terminator,
+    TpChannelGroupChangeReason reason,
+    const gchar *text,
     gpointer user_data);
 static void session_new_content_cb (GabbleJingleSession *session,
     GabbleJingleContent *c, gpointer user_data);
 static void create_stream_from_content (GabbleMediaChannel *chan,
-    GabbleJingleContent *c);
+    GabbleJingleContent *c, gboolean initial);
 static gboolean contact_is_media_capable (GabbleMediaChannel *chan, TpHandle peer,
     gboolean *wait, GError **error);
 static void stream_creation_data_cancel (gpointer p, gpointer unused);
@@ -228,7 +232,7 @@ create_initial_streams (GabbleMediaChannel *chan)
           g_assert_not_reached ();
         }
 
-      create_stream_from_content (chan, c);
+      create_stream_from_content (chan, c, TRUE);
     }
 
   DEBUG ("initial_audio: %s, initial_video: %s",
@@ -328,8 +332,15 @@ gabble_media_channel_constructor (GType type, guint n_props,
       TP_CHANNEL_GROUP_CHANGE_REASON_NONE);
   tp_intset_destroy (set);
 
-  /* We implement the 0.17.6 properties correctly */
-  tp_group_mixin_change_flags (obj, TP_CHANNEL_GROUP_FLAG_PROPERTIES, 0);
+  /* We implement the 0.17.6 properties correctly, and can include a message
+   * when ending a call.
+   */
+  tp_group_mixin_change_flags (obj,
+      TP_CHANNEL_GROUP_FLAG_PROPERTIES |
+      TP_CHANNEL_GROUP_FLAG_MESSAGE_REMOVE |
+      TP_CHANNEL_GROUP_FLAG_MESSAGE_REJECT |
+      TP_CHANNEL_GROUP_FLAG_MESSAGE_RESCIND,
+      0);
 
   /* Set up Google relay related properties */
   jf = priv->conn->jingle_factory;
@@ -405,6 +416,32 @@ gabble_media_channel_constructor (GType type, guint n_props,
         }
     }
 
+  /* If this is a Google session, let's set ImmutableStreams */
+  if (priv->session != NULL)
+    {
+      JingleDialect d = gabble_jingle_session_get_dialect (priv->session);
+
+      priv->immutable_streams = JINGLE_IS_GOOGLE_DIALECT (d);
+    }
+  /* If there's no session yet, but we know who the peer will be, and we have
+   * presence for them, we can set ImmutableStreams using the same algorithm as
+   * for old-style capabilities.  If we don't know who the peer will be, then
+   * the client is using an old calling convention and doesn't need to know
+   * this.
+   */
+  else if (priv->initial_peer != 0)
+    {
+      GabblePresence *presence = gabble_presence_cache_get (
+          priv->conn->presence_cache, priv->initial_peer);
+      TpChannelMediaCapabilities flags = 0;
+
+      if (presence != NULL)
+          _gabble_media_channel_caps_to_typeflags (presence->caps);
+
+      if (flags & TP_CHANNEL_MEDIA_CAPABILITY_IMMUTABLE_STREAMS)
+        priv->immutable_streams = TRUE;
+    }
+
   return obj;
 }
 
@@ -509,6 +546,9 @@ gabble_media_channel_get_property (GObject    *object,
               TP_IFACE_CHANNEL, "InitiatorID",
               TP_IFACE_CHANNEL, "Requested",
               TP_IFACE_CHANNEL, "Interfaces",
+              TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA, "ImmutableStreams",
+              TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA, "InitialAudio",
+              TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA, "InitialVideo",
               GABBLE_IFACE_CHANNEL_TYPE_STREAMED_MEDIA_FUTURE, "InitialAudio",
               GABBLE_IFACE_CHANNEL_TYPE_STREAMED_MEDIA_FUTURE, "InitialVideo",
               NULL));
@@ -522,6 +562,9 @@ gabble_media_channel_get_property (GObject    *object,
     case PROP_INITIAL_VIDEO:
       g_value_set_boolean (value, priv->initial_video);
       break;
+    case PROP_IMMUTABLE_STREAMS:
+      g_value_set_boolean (value, priv->immutable_streams);
+      break;
     default:
       param_name = g_param_spec_get_name (pspec);
 
@@ -644,6 +687,12 @@ gabble_media_channel_class_init (GabbleMediaChannelClass *gabble_media_channel_c
       { NULL }
   };
   static TpDBusPropertiesMixinPropImpl streamed_media_props[] = {
+      { "ImmutableStreams", "immutable-streams", NULL },
+      { "InitialAudio", "initial-audio", NULL },
+      { "InitialVideo", "initial-video", NULL },
+      { NULL }
+  };
+  static TpDBusPropertiesMixinPropImpl streamed_media_future_props[] = {
       { "InitialAudio", "initial-audio", NULL },
       { "InitialVideo", "initial-video", NULL },
       { NULL }
@@ -654,11 +703,16 @@ gabble_media_channel_class_init (GabbleMediaChannelClass *gabble_media_channel_c
         NULL,
         channel_props,
       },
-      { GABBLE_IFACE_CHANNEL_TYPE_STREAMED_MEDIA_FUTURE,
+      { TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
         tp_dbus_properties_mixin_getter_gobject_properties,
         NULL,
         streamed_media_props,
       },
+      { GABBLE_IFACE_CHANNEL_TYPE_STREAMED_MEDIA_FUTURE,
+        tp_dbus_properties_mixin_getter_gobject_properties,
+        NULL,
+        streamed_media_future_props,
+      },
       { NULL }
   };
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_media_channel_class);
@@ -797,6 +851,13 @@ gabble_media_channel_class_init (GabbleMediaChannelClass *gabble_media_channel_c
   g_object_class_install_property (object_class, PROP_INITIAL_VIDEO,
       param_spec);
 
+  param_spec = g_param_spec_boolean ("immutable-streams", "ImmutableStreams",
+      "Whether the set of streams on this channel are fixed once requested",
+      FALSE,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_IMMUTABLE_STREAMS,
+      param_spec);
+
   tp_properties_mixin_class_init (object_class,
       G_STRUCT_OFFSET (GabbleMediaChannelClass, properties_class),
       channel_property_signatures, NUM_CHAN_PROPS, NULL);
@@ -920,7 +981,7 @@ gabble_media_channel_close (GabbleMediaChannel *self)
 
       if (priv->session != NULL)
         gabble_jingle_session_terminate (priv->session,
-            TP_CHANNEL_GROUP_CHANGE_REASON_NONE, NULL);
+            TP_CHANNEL_GROUP_CHANGE_REASON_NONE, NULL, NULL);
 
       tp_svc_channel_emit_closed (self);
     }
@@ -1477,7 +1538,12 @@ CHOOSE_TRANSPORT:
   /* We prefer gtalk-p2p to ice, because it can use tcp and https relays (if
    * available). */
 
-  if (gabble_presence_resource_has_caps (presence, resource,
+  if (*dialect == JINGLE_DIALECT_GTALK4 || *dialect == JINGLE_DIALECT_GTALK3)
+    {
+      /* the GTalk dialects only support google p2p as transport protocol. */
+      *transport_ns = NS_GOOGLE_TRANSPORT_P2P;
+    }
+  else if (gabble_presence_resource_has_caps (presence, resource,
         PRESENCE_CAP_GOOGLE_TRANSPORT_P2P))
     {
       *transport_ns = NS_GOOGLE_TRANSPORT_P2P;
@@ -1492,14 +1558,6 @@ CHOOSE_TRANSPORT:
     {
       *transport_ns = NS_JINGLE_TRANSPORT_RAWUDP;
     }
-  else if (*dialect == JINGLE_DIALECT_GTALK4
-      || *dialect == JINGLE_DIALECT_GTALK3)
-    {
-      /* (Some) GTalk clients don't advertise gtalk-p2p, though
-       * they support it. If we know it's GTalk and there's no
-       * transport, we can assume it also. */
-      *transport_ns = NS_GOOGLE_TRANSPORT_P2P;
-    }
 
   if (*transport_ns == NULL)
       return NULL;
@@ -2068,6 +2126,22 @@ contact_is_media_capable (GabbleMediaChannel *chan,
       conn, TP_HANDLE_TYPE_CONTACT);
   gboolean wait_ = FALSE;
 
+  presence = gabble_presence_cache_get (priv->conn->presence_cache, peer);
+
+  caps = PRESENCE_CAP_GOOGLE_VOICE | PRESENCE_CAP_GOOGLE_VOICE |
+    PRESENCE_CAP_JINGLE015 |
+    PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO |
+    PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO |
+    PRESENCE_CAP_JINGLE_RTP |
+    PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO |
+    PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO;
+
+  if (presence != NULL && (presence->caps & caps) != 0)
+    return TRUE;
+
+  /* Okay, they're not capable (yet). Let's figure out whether we should wait,
+   * and return an appropriate error.
+   */
   if (gabble_presence_cache_caps_pending (priv->conn->presence_cache, peer))
     {
       DEBUG ("caps are pending for peer %u", peer);
@@ -2082,30 +2156,16 @@ contact_is_media_capable (GabbleMediaChannel *chan,
   if (wait != NULL)
     *wait = wait_;
 
-  caps = PRESENCE_CAP_GOOGLE_VOICE | PRESENCE_CAP_GOOGLE_VOICE |
-    PRESENCE_CAP_JINGLE_RTP |
-    PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO |
-    PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO;
-
-  presence = gabble_presence_cache_get (priv->conn->presence_cache, peer);
-
   if (presence == NULL)
-    {
-      g_set_error (error, TP_ERRORS, TP_ERROR_OFFLINE,
-          "contact %d (%s) has no presence available", peer,
-          tp_handle_inspect (contact_handles, peer));
-      return FALSE;
-    }
+    g_set_error (error, TP_ERRORS, TP_ERROR_OFFLINE,
+        "contact %d (%s) has no presence available", peer,
+        tp_handle_inspect (contact_handles, peer));
+  else
+    g_set_error (error, TP_ERRORS, TP_ERROR_NOT_CAPABLE,
+        "contact %d (%s) doesn't have sufficient media caps", peer,
+        tp_handle_inspect (contact_handles, peer));
 
-  if ((presence->caps & caps) == 0)
-    {
-      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_CAPABLE,
-          "contact %d (%s) doesn't have sufficient media caps", peer,
-          tp_handle_inspect (contact_handles, peer));
-      return FALSE;
-    }
-
-  return TRUE;
+  return FALSE;
 }
 
 static gboolean
@@ -2240,7 +2300,8 @@ gabble_media_channel_remove_member (GObject *obj,
       /* Terminate can fail if the UI provides a reason that makes no sense,
        * like Invited.
        */
-      if (!gabble_jingle_session_terminate (priv->session, reason, error))
+      if (!gabble_jingle_session_terminate (priv->session, reason, message,
+              error))
         {
           g_object_unref (chan);
           return FALSE;
@@ -2273,6 +2334,7 @@ static void
 session_terminated_cb (GabbleJingleSession *session,
                        gboolean local_terminator,
                        TpChannelGroupChangeReason reason,
+                       const gchar *text,
                        gpointer user_data)
 {
   GabbleMediaChannel *channel = (GabbleMediaChannel *) user_data;
@@ -2302,7 +2364,7 @@ session_terminated_cb (GabbleJingleSession *session,
   tp_intset_add (set, peer);
 
   tp_group_mixin_change_members ((GObject *) channel,
-      "", NULL, set, NULL, NULL, terminator, reason);
+      text, NULL, set, NULL, NULL, terminator, reason);
 
   tp_intset_destroy (set);
 
@@ -2423,6 +2485,7 @@ stream_error_cb (GabbleMediaStream *stream,
 {
   GabbleMediaChannelPrivate *priv = chan->priv;
   GabbleJingleMediaRtp *c;
+  GList *contents;
   guint id;
 
   /* emit signal */
@@ -2430,7 +2493,10 @@ stream_error_cb (GabbleMediaStream *stream,
   tp_svc_channel_type_streamed_media_emit_stream_error (chan, id, errno,
       message);
 
-  if (gabble_jingle_session_can_modify_contents (priv->session))
+  contents = gabble_jingle_session_get_contents (priv->session);
+
+  if (gabble_jingle_session_can_modify_contents (priv->session) &&
+      g_list_length (contents) > 1)
     {
       /* remove stream from session (removal will be signalled
        * so we can dispose of the stream)
@@ -2441,14 +2507,17 @@ stream_error_cb (GabbleMediaStream *stream,
     }
   else
     {
-      /* We can't remove the content; let's terminate the call. (The
-       * alternative is to carry on the call with only audio/video, which will
-       * look or sound bad to the Google Talk-using peer.)
+      /* We can't remove the content, or it's the only one left; let's
+       * terminate the call. (The alternative is to carry on the call with
+       * only audio/video, which will look or sound bad to the Google
+       * Talk-using peer.)
        */
       DEBUG ("Terminating call in response to stream error");
       gabble_jingle_session_terminate (priv->session,
-          TP_CHANNEL_GROUP_CHANGE_REASON_ERROR, NULL);
+          TP_CHANNEL_GROUP_CHANGE_REASON_ERROR, message, NULL);
     }
+
+  g_list_free (contents);
 }
 
 static void
@@ -2555,7 +2624,8 @@ construct_stream (GabbleMediaChannel *chan,
                   GabbleJingleContent *c,
                   const gchar *name,
                   const gchar *nat_traversal,
-                  const GPtrArray *relays)
+                  const GPtrArray *relays,
+                  gboolean initial)
 {
   GObject *chan_o = (GObject *) chan;
   GabbleMediaChannelPrivate *priv = chan->priv;
@@ -2612,6 +2682,17 @@ construct_stream (GabbleMediaChannel *chan,
   gabble_signal_connect_weak (stream, "notify::combined-direction",
       (GCallback) stream_direction_changed_cb, chan_o);
 
+  if (initial)
+    {
+      /* If we accepted the call, then automagically accept the initial streams
+       * when they pop up */
+      if (tp_handle_set_is_member (chan->group.members,
+          chan->group.self_handle))
+        {
+          gabble_media_stream_accept_pending_local_send (stream);
+        }
+    }
+
   /* emit StreamAdded */
   mtype = gabble_media_stream_get_media_type (stream);
 
@@ -2686,7 +2767,8 @@ construct_stream_later_cb (gpointer user_data)
   StreamCreationData *d = user_data;
 
   if (d->content != NULL && d->self != NULL)
-    construct_stream (d->self, d->content, d->name, d->nat_traversal, NULL);
+    construct_stream (d->self, d->content, d->name, d->nat_traversal, NULL,
+      d->initial);
 
   return FALSE;
 }
@@ -2698,7 +2780,8 @@ google_relay_session_cb (GPtrArray *relays,
   StreamCreationData *d = user_data;
 
   if (d->content != NULL && d->self != NULL)
-    construct_stream (d->self, d->content, d->name, d->nat_traversal, relays);
+    construct_stream (d->self, d->content, d->name, d->nat_traversal, relays,
+      d->initial);
 
   stream_creation_data_free (d);
 }
@@ -2744,7 +2827,8 @@ content_removed_cb (GabbleJingleContent *content,
 
 static void
 create_stream_from_content (GabbleMediaChannel *self,
-                            GabbleJingleContent *c)
+                            GabbleJingleContent *c,
+                            gboolean initial)
 {
   gchar *name;
   StreamCreationData *d;
@@ -2765,6 +2849,7 @@ create_stream_from_content (GabbleMediaChannel *self,
   d->self = self;
   d->name = name;
   d->content = g_object_ref (c);
+  d->initial = initial;
 
   g_object_add_weak_pointer (G_OBJECT (d->self), (gpointer *) &d->self);
 
@@ -2811,13 +2896,24 @@ session_new_content_cb (GabbleJingleSession *session,
 
   DEBUG ("called");
 
-  create_stream_from_content (chan, c);
+  create_stream_from_content (chan, c, FALSE);
 }
 
 TpChannelMediaCapabilities
 _gabble_media_channel_caps_to_typeflags (GabblePresenceCapabilities caps)
 {
   TpChannelMediaCapabilities typeflags = 0;
+  GabblePresenceCapabilities any_transport =
+      PRESENCE_CAP_GOOGLE_TRANSPORT_P2P |
+      PRESENCE_CAP_JINGLE_TRANSPORT_RAWUDP |
+      PRESENCE_CAP_JINGLE_TRANSPORT_ICEUDP;
+  GabblePresenceCapabilities jingle_audio =
+      PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO |
+      PRESENCE_CAP_JINGLE_RTP_AUDIO;
+  GabblePresenceCapabilities jingle_video =
+      PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO |
+      PRESENCE_CAP_JINGLE_RTP_VIDEO;
+  gboolean just_google, one_media_type;
 
   /* this is intentionally asymmetric to the previous function - we don't
    * require the other end to advertise the GTalk-P2P transport capability
@@ -2825,19 +2921,23 @@ _gabble_media_channel_caps_to_typeflags (GabblePresenceCapabilities caps)
    * implied Google session and GTalk-P2P */
 
   if ((caps & PRESENCE_CAP_GOOGLE_VOICE) ||
-      ((caps & ( PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO
-               | PRESENCE_CAP_JINGLE_RTP_AUDIO
-               )) &&
-       (caps & PRESENCE_CAP_GOOGLE_TRANSPORT_P2P)))
+      ((caps & any_transport) && (caps & jingle_audio)))
         typeflags |= TP_CHANNEL_MEDIA_CAPABILITY_AUDIO;
 
-  if ( (caps & PRESENCE_CAP_GOOGLE_VIDEO) ||
-      ((caps & ( PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO
-               | PRESENCE_CAP_JINGLE_RTP_VIDEO
-               )) &&
-       (caps & PRESENCE_CAP_GOOGLE_TRANSPORT_P2P)))
+  if ((caps & PRESENCE_CAP_GOOGLE_VIDEO) ||
+      ((caps & any_transport) && (caps & jingle_video)))
         typeflags |= TP_CHANNEL_MEDIA_CAPABILITY_VIDEO;
 
+  /* If this contact only supports Google Talk, or supports exactly one media
+   * type, set Immutable_Streams. */
+  just_google = (caps & (PRESENCE_CAP_GOOGLE_VOICE | PRESENCE_CAP_GOOGLE_VIDEO))
+      && !(caps & (PRESENCE_CAP_JINGLE_RTP | PRESENCE_CAP_JINGLE015));
+  one_media_type = (typeflags == TP_CHANNEL_MEDIA_CAPABILITY_AUDIO)
+      || (typeflags == TP_CHANNEL_MEDIA_CAPABILITY_VIDEO);
+
+  if (just_google || one_media_type)
+    typeflags |= TP_CHANNEL_MEDIA_CAPABILITY_IMMUTABLE_STREAMS;
+
   return typeflags;
 }
 
