@@ -37,9 +37,14 @@
 #include "request-pipeline.h"
 #include "util.h"
 
-#define DEFAULT_REQUEST_TIMEOUT 180
+static guint default_request_timeout = 180;
 #define VCARD_CACHE_ENTRY_TTL 60
 
+/* When the server reply with XMPP_ERROR_RESOURCE_CONSTRAINT, wait
+ * request_wait_delay seconds before allowing a vCard request to be sent to
+ * the same recipient */
+static guint request_wait_delay = 5 * 60;
+
 static const gchar *NO_ALIAS = "none";
 
 /* signal enum */
@@ -146,6 +151,12 @@ struct _GabbleVCardCacheEntry
   /* List of (GabbleVCardManagerRequest *) borrowed from priv->requests */
   GSList *pending_requests;
 
+  /* When requests for this entry receive an error of type "wait", we suspend
+   * further requests and retry again after request_wait_delay seconds.
+   * 0 if not suspended.
+   */
+  guint suspended_timer_id;
+
   /* VCard node for this entry (owned reference), or NULL if there's no node */
   LmMessageNode *vcard_node;
 
@@ -175,6 +186,8 @@ static void cache_entry_free (void *data);
 static gint cache_entry_compare (gconstpointer a, gconstpointer b);
 static void manager_patch_vcard (
     GabbleVCardManager *self, LmMessageNode *vcard_node);
+static void request_send (GabbleVCardManagerRequest *request,
+    guint timeout);
 
 static void
 gabble_vcard_manager_init (GabbleVCardManager *obj)
@@ -403,6 +416,10 @@ cache_entry_attempt_to_free (GabbleVCardCacheEntry *entry)
       return;
     }
 
+  /* If there is a suspended request, it must be in entry-> pending_requests
+   */
+  g_assert (entry->suspended_timer_id == 0);
+
   if (entry->handle == base->self_handle)
     {
       /* if we do have some pending edits, we should also have
@@ -481,6 +498,12 @@ disconnect_entry_foreach (gpointer handle, gpointer value, gpointer unused)
   GError err = { TP_ERRORS, TP_ERROR_DISCONNECTED, "Connection closed" };
   GabbleVCardCacheEntry *entry = value;
 
+  if (entry->suspended_timer_id)
+    {
+      g_source_remove (entry->suspended_timer_id);
+      entry->suspended_timer_id = 0;
+    }
+
   cache_entry_complete_requests (entry, &err);
 
   if (entry->pipeline_item)
@@ -530,7 +553,7 @@ gabble_vcard_manager_finalize (GObject *object)
   G_OBJECT_CLASS (gabble_vcard_manager_parent_class)->finalize (object);
 }
 
-static gchar *
+gchar *
 vcard_get_avatar_sha1 (LmMessageNode *vcard)
 {
   gchar *sha1;
@@ -708,15 +731,17 @@ static gboolean
 timeout_request (gpointer data)
 {
   GabbleVCardManagerRequest *request = (GabbleVCardManagerRequest *) data;
-  GError err = { GABBLE_VCARD_MANAGER_ERROR,
-      GABBLE_VCARD_MANAGER_ERROR_TIMEOUT, "Request timed out" };
 
   g_return_val_if_fail (data != NULL, FALSE);
   DEBUG ("Request %p timed out, notifying callback %p",
          request, request->callback);
 
   request->timer_id = 0;
-  complete_one_request (request, NULL, &err);
+
+  /* The pipeline machinery will call our callback with the error "canceled"
+   */
+  gabble_request_pipeline_item_cancel (request->entry->pipeline_item);
+
   return FALSE;
 }
 
@@ -890,8 +915,21 @@ replace_reply_cb (GabbleConnection *conn,
         }
     }
 
-  /* If we've received more edit requests in the meantime, send them off. */
-  manager_patch_vcard (self, node);
+  if (error != NULL)
+    {
+      if (priv->edits != NULL)
+        {
+          /* All the requests for these edits have just been cancelled. */
+          g_hash_table_destroy (priv->edits);
+          priv->edits = NULL;
+        }
+    }
+  else
+    {
+      /* If we've received more edit requests in the meantime, send them off.
+       */
+      manager_patch_vcard (self, node);
+    }
 }
 
 static void
@@ -993,7 +1031,7 @@ manager_patch_vcard (GabbleVCardManager *self,
   priv->patched_vcard = lm_message_node_ref (patched_vcard);
 
   priv->edit_pipeline_item = gabble_request_pipeline_enqueue (
-      priv->connection->req_pipeline, msg, DEFAULT_REQUEST_TIMEOUT,
+      priv->connection->req_pipeline, msg, default_request_timeout,
       replace_reply_cb, self);
 
   lm_message_unref (msg);
@@ -1011,6 +1049,18 @@ manager_patch_vcard (GabbleVCardManager *self,
     }
 }
 
+static gboolean
+suspended_request_timeout_cb (gpointer data)
+{
+  GabbleVCardManagerRequest *request = data;
+
+  /* Send the request again */
+  request->entry->suspended_timer_id = 0;
+  request_send (request, request->timeout);
+
+  return FALSE;
+}
+
 /* Called when a GET request in the pipeline has either succeeded or failed. */
 static void
 pipeline_reply_cb (GabbleConnection *conn,
@@ -1018,7 +1068,8 @@ pipeline_reply_cb (GabbleConnection *conn,
                    gpointer user_data,
                    GError *error)
 {
-  GabbleVCardCacheEntry *entry = user_data;
+  GabbleVCardManagerRequest *request = user_data;
+  GabbleVCardCacheEntry *entry = request->entry;
   GabbleVCardManager *self = GABBLE_VCARD_MANAGER (entry->manager);
   GabbleVCardManagerPrivate *priv = self->priv;
   TpBaseConnection *base = (TpBaseConnection *) conn;
@@ -1031,11 +1082,42 @@ pipeline_reply_cb (GabbleConnection *conn,
   g_assert (tp_handle_is_valid (contact_repo, entry->handle, NULL));
 
   g_assert (entry->pipeline_item != NULL);
+  g_assert (entry->suspended_timer_id == 0);
 
   entry->pipeline_item = NULL;
 
   if (error)
     {
+      /* First, handle the error "wait": suspend the request and replay it
+       * later */
+      LmMessageNode *error_node = NULL;
+      GabbleXmppError xmpp_error = XMPP_ERROR_UNDEFINED_CONDITION;
+      GabbleXmppErrorType error_type = XMPP_ERROR_UNDEFINED_CONDITION;
+
+      /* FIXME: add a helper in error.c to extract the type, error, and message
+       *        from an XMPP stanza.
+       */
+      if (reply_msg != NULL)
+        error_node = lm_message_node_get_child (reply_msg->node, "error");
+
+      if (error_node != NULL)
+        xmpp_error = gabble_xmpp_error_from_node (error_node, &error_type);
+
+      if (error_type == XMPP_ERROR_TYPE_WAIT)
+        {
+          DEBUG ("Retrieving %u's vCard returned a temporary <%s/> error; "
+              "trying againg in %u seconds", entry->handle,
+              gabble_xmpp_error_string (xmpp_error), request_wait_delay);
+
+          g_source_remove (request->timer_id);
+          request->timer_id = 0;
+
+          entry->suspended_timer_id = g_timeout_add_seconds (
+              request_wait_delay, suspended_request_timeout_cb, request);
+
+          return;
+        }
+
       /* If request for our own vCard failed, and we do have
        * pending edits to make, cancel those and return error
        * to the user */
@@ -1107,22 +1189,32 @@ notify_delete_request (gpointer data, GObject *obj)
 }
 
 static void
-cache_entry_ensure_queued (GabbleVCardCacheEntry *entry, guint timeout)
+request_send (GabbleVCardManagerRequest *request, guint timeout)
 {
+  GabbleVCardCacheEntry *entry = request->entry;
   GabbleConnection *conn = entry->manager->priv->connection;
   TpBaseConnection *base = (TpBaseConnection *) conn;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (base,
       TP_HANDLE_TYPE_CONTACT);
 
+  g_assert (request->timer_id == 0);
+
   if (entry->pipeline_item)
     {
       DEBUG ("adding to cache entry %p with <iq> already pending", entry);
     }
+  else if (entry->suspended_timer_id != 0)
+    {
+      DEBUG ("adding to cache entry %p with <iq> suspended", entry);
+    }
   else
     {
       const char *jid;
       LmMessage *msg;
 
+      request->timer_id =
+          g_timeout_add_seconds (request->timeout, timeout_request, request);
+
       if (entry->handle == base->self_handle)
         {
           DEBUG ("Cache entry %p is my own, not setting @to", entry);
@@ -1142,7 +1234,7 @@ cache_entry_ensure_queued (GabbleVCardCacheEntry *entry, guint timeout)
           NULL);
 
       entry->pipeline_item = gabble_request_pipeline_enqueue (
-          conn->req_pipeline, msg, timeout, pipeline_reply_cb, entry);
+          conn->req_pipeline, msg, timeout, pipeline_reply_cb, request);
 
       lm_message_unref (msg);
 
@@ -1155,6 +1247,9 @@ cache_entry_ensure_queued (GabbleVCardCacheEntry *entry, guint timeout)
  *
  * The callback may be NULL if you just want the side-effect of this
  * operation, which is to update the cached alias.
+ *
+ * FIXME: the timeout is not always obeyed when there is already a request
+ *        on the same handle. It should perhaps be removed.
  */
 GabbleVCardManagerRequest *
 gabble_vcard_manager_request (GabbleVCardManager *self,
@@ -1175,7 +1270,7 @@ gabble_vcard_manager_request (GabbleVCardManager *self,
   g_assert (entry->vcard_node == NULL);
 
   if (timeout == 0)
-    timeout = DEFAULT_REQUEST_TIMEOUT;
+    timeout = default_request_timeout;
 
   request = g_slice_new0 (GabbleVCardManagerRequest);
   DEBUG ("Created request %p to retrieve <%u>'s vCard", request, handle);
@@ -1192,9 +1287,7 @@ gabble_vcard_manager_request (GabbleVCardManager *self,
   request->entry->pending_requests = g_slist_prepend
       (request->entry->pending_requests, request);
 
-  request->timer_id =
-      g_timeout_add_seconds (timeout, timeout_request, request);
-  cache_entry_ensure_queued (request->entry, timeout);
+  request_send (request, timeout);
   return request;
 }
 
@@ -1406,3 +1499,16 @@ gabble_vcard_manager_has_cached_alias (GabbleVCardManager *self,
 
   return p != NULL;
 }
+
+/* For unit tests only */
+void
+gabble_vcard_manager_set_suspend_reply_timeout (guint timeout)
+{
+  request_wait_delay = timeout;
+}
+
+void
+gabble_vcard_manager_set_default_request_timeout (guint timeout)
+{
+  default_request_timeout = timeout;
+}
