@@ -8,6 +8,7 @@ import os
 import hashlib
 import sys
 import random
+import traceback
 
 import ns
 import constants as cs
@@ -335,30 +336,6 @@ def make_stream(event_func, authenticator=None, protocol=None, port=4242, resour
     port = reactor.listenTCP(port, factory)
     return (stream, port)
 
-def install_colourer():
-    def red(s):
-        return '\x1b[31m%s\x1b[0m' % s
-
-    def green(s):
-        return '\x1b[32m%s\x1b[0m' % s
-
-    patterns = {
-        'handled': green,
-        'not handled': red,
-        }
-
-    class Colourer:
-        def __init__(self, fh, patterns):
-            self.fh = fh
-            self.patterns = patterns
-
-        def write(self, s):
-            f = self.patterns.get(s, lambda x: x)
-            self.fh.write(f(s))
-
-    sys.stdout = Colourer(sys.stdout, patterns)
-    return sys.stdout
-
 def disconnect_conn(q, conn, stream, expected=[]):
     conn.Disconnect()
 
@@ -368,14 +345,14 @@ def disconnect_conn(q, conn, stream, expected=[]):
     events = q.expect_many(*tmp)
     return events[:-1]
 
-def exec_test_deferred(funs, params, protocol=None, timeout=None,
+def exec_test_deferred(fun, params, protocol=None, timeout=None,
                         authenticator=None):
     # hack to ease debugging
     domish.Element.__repr__ = domish.Element.toXml
     colourer = None
 
-    if sys.stdout.isatty():
-        colourer = install_colourer()
+    if sys.stdout.isatty() or 'CHECK_FORCE_COLOR' in os.environ:
+        colourer = servicetest.install_colourer()
 
     queue = servicetest.IteratingEventQueue(timeout)
     queue.verbose = (
@@ -383,7 +360,7 @@ def exec_test_deferred(funs, params, protocol=None, timeout=None,
         or '-v' in sys.argv)
 
     bus = dbus.SessionBus()
-    # conn = make_connection(bus, queue.append, params)
+    conn = make_connection(bus, queue.append, params)
     resource = params.get('resource') if params is not None else None
     (stream, port) = make_stream(queue.append, protocol=protocol,
         authenticator=authenticator, resource=resource)
@@ -391,11 +368,8 @@ def exec_test_deferred(funs, params, protocol=None, timeout=None,
     error = None
 
     try:
-        for f in funs:
-            conn = make_connection(bus, queue.append, params)
-            f(queue, bus, conn, stream)
+        fun(queue, bus, conn, stream)
     except Exception, e:
-        import traceback
         traceback.print_exc()
         error = e
 
@@ -416,15 +390,11 @@ def exec_test_deferred(funs, params, protocol=None, timeout=None,
         # Connection has already been disconnected
         pass
 
-def exec_tests(funs, params=None, protocol=None, timeout=None,
-               authenticator=None):
-    reactor.callWhenRunning(
-        exec_test_deferred, funs, params, protocol, timeout, authenticator)
-    reactor.run()
-
 def exec_test(fun, params=None, protocol=None, timeout=None,
               authenticator=None):
-  exec_tests([fun], params, protocol, timeout, authenticator)
+    reactor.callWhenRunning(
+        exec_test_deferred, fun, params, protocol, timeout, authenticator)
+    reactor.run()
 
 # Useful routines for server-side vCard handling
 current_vcard = domish.Element(('vcard-temp', 'vCard'))
@@ -514,7 +484,8 @@ def elem_iq(server, type, **kw):
 
     return iq
 
-def make_presence(_from, to='test@localhost', type=None, status=None, caps=None):
+def make_presence(_from, to='test@localhost', type=None, show=None,
+        status=None, caps=None, photo=None):
     presence = domish.Element((None, 'presence'))
     presence['from'] = _from
     presence['to'] = to
@@ -522,6 +493,9 @@ def make_presence(_from, to='test@localhost', type=None, status=None, caps=None)
     if type is not None:
         presence['type'] = type
 
+    if show is not None:
+        presence.addElement('show', content=show)
+
     if status is not None:
         presence.addElement('status', content=status)
 
@@ -530,6 +504,11 @@ def make_presence(_from, to='test@localhost', type=None, status=None, caps=None)
         for key,value in caps.items():
             cel[key] = value
 
+    # <x xmlns="vcard-temp:x:update"><photo>4a1...</photo></x>
+    if photo is not None:
+        x = presence.addElement((ns.VCARD_TEMP_UPDATE, 'x'))
+        x.addElement('photo').addContent(photo)
+
     return presence
 
 def expect_list_channel(q, bus, conn, name, contacts, lp_contacts=[],
