@@ -21,19 +21,18 @@ def test(q, bus, conn, stream):
 
     conn.Connect()
 
-    q.expect('dbus-signal', signal='StatusChanged', args=[1, 1])
+    q.expect('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTING, cs.CSR_REQUESTED])
 
     q.expect('stream-authenticated')
     q.expect('dbus-signal', signal='PresenceUpdate',
         args=[{1L: (0L, {u'available': {}})}])
-    q.expect('dbus-signal', signal='StatusChanged', args=[0, 1])
+    q.expect('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED])
 
     run_test(q, bus, conn, stream, jt, True)
     run_test(q, bus, conn, stream, jt2, False)
 
-    conn.Disconnect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[2, 1])
-
 def run_test(q, bus, conn, stream, jt, request_before_presence):
     """
     Requests streams on a media channel to jt.remote_jid, either before their
