@@ -4,17 +4,18 @@ test gadget invitation when an activity becomes public
 
 import dbus
 
-from servicetest import call_async, EventPattern, sync_dbus
-from gabbletest import exec_test, make_result_iq, acknowledge_iq, sync_stream
+from servicetest import call_async, EventPattern
+from gabbletest import exec_test, make_result_iq, acknowledge_iq, sync_stream,\
+    make_muc_presence, disconnect_conn
+import constants as cs
 
-from twisted.words.xish import domish, xpath
-from twisted.words.protocols.jabber.client import IQ
+from twisted.words.xish import xpath
 
 from util import announce_gadget, gadget_publish
 import ns
 
 def join_channel(name, q, conn, stream):
-    call_async(q, conn, 'RequestHandles', 2, [name])
+    call_async(q, conn, 'RequestHandles', cs.HT_ROOM, [name])
 
     # announce conference service
     event = q.expect('stream-iq', to='conference.localhost', query_ns=ns.DISCO_INFO)
@@ -26,18 +27,12 @@ def join_channel(name, q, conn, stream):
     event = q.expect('dbus-return', method='RequestHandles')
     handles = event.value[0]
 
-    call_async(q, conn, 'RequestChannel',
-            'org.freedesktop.Telepathy.Channel.Type.Text', 2, handles[0], True)
+    call_async(q, conn, 'RequestChannel', cs.CHANNEL_TYPE_TEXT, cs.HT_ROOM,
+        handles[0], True)
 
     event = q.expect('stream-presence', to='myroom@conference.localhost/test')
     # Send presence for own membership of room.
-    presence = domish.Element((None, 'presence'))
-    presence['from'] = 'myroom@conference.localhost/test'
-    x = presence.addElement((ns.MUC_USER, 'x'))
-    item = x.addElement('item')
-    item['affiliation'] = 'none'
-    item['role'] = 'participant'
-    stream.send(presence)
+    stream.send(make_muc_presence('none', 'participant', 'myroom@conference.localhost', 'test'))
 
     event = q.expect('dbus-return', method='RequestChannel')
     return handles[0], event.value[0]
@@ -46,7 +41,8 @@ def test(q, bus, conn, stream):
     conn.Connect()
 
     _, iq_event, disco_event = q.expect_many(
-        EventPattern('dbus-signal', signal='StatusChanged', args=[0, 1]),
+        EventPattern('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
         EventPattern('stream-iq', to=None, query_ns='vcard-temp',
             query_name='vCard'),
         EventPattern('stream-iq', to='localhost', query_ns=ns.DISCO_ITEMS))
@@ -109,14 +105,8 @@ def test(q, bus, conn, stream):
     event = q.expect('dbus-return', method='SetProperties')
 
     # Gadget joins the room
-    presence = domish.Element((None, 'presence'))
-    presence['from'] = 'myroom@conference.localhost/inspector'
-    x = presence.addElement((ns.MUC_USER, 'x'))
-    item = x.addElement('item')
-    item['jid'] = 'gadget.localhost'
-    item['affiliation'] = 'none'
-    item['role'] = 'participant'
-    stream.send(presence)
+    stream.send(make_muc_presence('none', 'participant', 'myroom@conference.localhost',
+        'inspector', 'gadget.localhost'))
 
     muc = bus.get_object(conn.bus_name, room_path)
     muc_group = dbus.Interface(muc, "org.freedesktop.Telepathy.Channel.Interface.Group")
@@ -132,11 +122,8 @@ def test(q, bus, conn, stream):
     presence = simple_presence_iface.GetPresences([handle])
     assert presence[handle] == (7, 'unknown', '')
 
-    conn.Disconnect()
-
     # PEP activity update
-    event = q.expect('stream-iq')
-    acknowledge_iq(stream, event.stanza)
+    disconnect_conn(q, conn, stream, [EventPattern('stream-iq')])[0]
 
 if __name__ == '__main__':
     exec_test(test)
