@@ -7,12 +7,13 @@ import dbus
 from servicetest import call_async, EventPattern
 from gabbletest import exec_test, make_result_iq, acknowledge_iq
 
-from twisted.words.xish import domish, xpath
+from twisted.words.xish import xpath
 from twisted.words.protocols.jabber.client import IQ
 
 from util import (announce_gadget, properties_to_xml, parse_properties,
     create_gadget_message, close_view)
 import ns
+import constants as cs
 
 tp_name_prefix = 'org.freedesktop.Telepathy'
 olpc_name_prefix = 'org.laptop.Telepathy'
@@ -21,7 +22,8 @@ def test(q, bus, conn, stream):
     conn.Connect()
 
     _, iq_event, disco_event = q.expect_many(
-        EventPattern('dbus-signal', signal='StatusChanged', args=[0, 1]),
+        EventPattern('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
         EventPattern('stream-iq', to=None, query_ns='vcard-temp',
             query_name='vCard'),
         EventPattern('stream-iq', to='localhost', query_ns=ns.DISCO_ITEMS))
@@ -34,7 +36,7 @@ def test(q, bus, conn, stream):
 
     # Gadget was not announced yet
     call_async(q, requests_iface, 'CreateChannel',
-        { 'org.freedesktop.Telepathy.Channel.ChannelType':
+        { cs.CHANNEL_TYPE:
             'org.laptop.Telepathy.Channel.Type.BuddyView',
             'org.laptop.Telepathy.Channel.Interface.View.MaxSize': 5,
           })
@@ -88,8 +90,7 @@ def test(q, bus, conn, stream):
 
     # check if we can request Buddy views
     properties = conn.GetAll(
-        'org.freedesktop.Telepathy.Connection.Interface.Requests',
-        dbus_interface=dbus.PROPERTIES_IFACE)
+        cs.CONN_IFACE_REQUESTS, dbus_interface=dbus.PROPERTIES_IFACE)
 
     assert ({tp_name_prefix + '.Channel.ChannelType':
             olpc_name_prefix + '.Channel.Type.BuddyView'},
@@ -154,9 +155,7 @@ def test(q, bus, conn, stream):
     assert props['org.laptop.Telepathy.Channel.Type.BuddyView.Alias'] == ''
 
     # check org.freedesktop.Telepathy.Channel D-Bus properties
-    props = view1.GetAll(
-        'org.freedesktop.Telepathy.Channel',
-        dbus_interface=dbus.PROPERTIES_IFACE)
+    props = view1.GetAll(cs.CHANNEL, dbus_interface=dbus.PROPERTIES_IFACE)
 
     assert props['ChannelType'] == 'org.laptop.Telepathy.Channel.Type.BuddyView'
     assert 'org.laptop.Telepathy.Channel.Interface.View' in props['Interfaces']
@@ -179,7 +178,7 @@ def test(q, bus, conn, stream):
     assert props['Properties'] == {}
     assert props['Alias'] == ''
 
-    assert view1.GetChannelType(dbus_interface='org.freedesktop.Telepathy.Channel') ==\
+    assert view1.GetChannelType(dbus_interface=cs.CHANNEL) ==\
             'org.laptop.Telepathy.Channel.Type.BuddyView'
 
     event = q.expect('dbus-signal', signal='BuddiesChanged')
@@ -299,8 +298,7 @@ def test(q, bus, conn, stream):
     assert conn.InspectHandles(1, added)[0] == 'oscar@localhost'
 
     members = view1.Get(olpc_name_prefix + '.Channel.Interface.View',
-        'Buddies',
-        dbus_interface=dbus.PROPERTIES_IFACE)
+        'Buddies', dbus_interface=dbus.PROPERTIES_IFACE)
 
     members = sorted(conn.InspectHandles(1, members))
     assert sorted(members) == ['bob@localhost', 'charles@localhost',
@@ -324,8 +322,7 @@ def test(q, bus, conn, stream):
     assert conn.InspectHandles(1, [handle])[0] == 'bob@localhost'
 
     members = view1.Get(olpc_name_prefix + '.Channel.Interface.View',
-        'Buddies',
-        dbus_interface=dbus.PROPERTIES_IFACE)
+        'Buddies', dbus_interface=dbus.PROPERTIES_IFACE)
     members = sorted(conn.InspectHandles(1, members))
     assert sorted(members) == ['charles@localhost', 'oscar@localhost']
 
@@ -392,12 +389,12 @@ def test(q, bus, conn, stream):
 
     # View request without MaxSize property
     call_async(q, requests_iface, 'CreateChannel',
-        { 'org.freedesktop.Telepathy.Channel.ChannelType':
+        { cs.CHANNEL_TYPE:
             'org.laptop.Telepathy.Channel.Type.BuddyView',
           })
 
     event = q.expect('dbus-error', method='CreateChannel')
-    assert event.error.get_dbus_name() == 'org.freedesktop.Telepathy.Errors.InvalidArgument'
+    assert event.error.get_dbus_name() == cs.INVALID_ARGUMENT
 
     # test alias and properties search
     props = dbus.Dictionary({'color': '#AABBCC,#001122'}, signature='sv')
