@@ -3,28 +3,24 @@ Regression test for a bug where RequestChannel times out when requesting a
 group channel if the roster hasn't been received at the time of the call.
 """
 
-import dbus
-
 from gabbletest import exec_test, sync_stream
 from servicetest import sync_dbus, call_async
-
-HT_GROUP = 4
+import constants as cs
+import ns
 
 def test(q, bus, conn, stream):
     conn.Connect()
-    # q.expect('dbus-signal', signal='StatusChanged', args=[0, 1])
 
-    roster_event = q.expect('stream-iq', query_ns='jabber:iq:roster')
+    roster_event = q.expect('stream-iq', query_ns=ns.ROSTER)
     roster_event.stanza['type'] = 'result'
 
-    call_async(q, conn, "RequestHandles", HT_GROUP, ['test'])
+    call_async(q, conn, "RequestHandles", cs.HT_GROUP, ['test'])
 
     event = q.expect('dbus-return', method='RequestHandles')
     test_handle = event.value[0][0]
 
-    call_async(q, conn, 'RequestChannel',
-        'org.freedesktop.Telepathy.Channel.Type.ContactList', HT_GROUP,
-        test_handle, True)
+    call_async(q, conn, 'RequestChannel', cs.CHANNEL_TYPE_CONTACT_LIST,
+        cs.HT_GROUP, test_handle, True)
 
     # A previous incarnation of this test --- written with the intention that
     # RequestChannel would be called before the roster was received, to expose
@@ -47,7 +43,7 @@ def test(q, bus, conn, stream):
         event = q.expect('dbus-signal', signal='NewChannel')
         assert event.args[0] == path, (event.args, path)
         _, type, handle_type, handle, suppress_handler = event.args
-        if handle_type == HT_GROUP and handle == test_handle:
+        if handle_type == cs.HT_GROUP and handle == test_handle:
             break
 
 if __name__ == '__main__':
